/* 
NiceRF LoRa1276 Module Arduino NANO Clone V3

NANO        LoRa1276
D11 MOSI    6  MOSI
D12 MISO    5  MISO
D13 SCK     4  SCK
D10         7  NSS

by absolutelyautomation.com

*/

// using SPI library:
#include <SPI.h>
#include <loraIno.h>
#include <AES.h>
/*
// This information is inside of loraIno.h
#define MOSI   11
#define MISO   12
#define SCK    13
#define SS     10

#define NRESET 7
#define TXEN   9
#define RXEN   8
#define LED1   A4
#define LED2   A5
*/

#define WITHOUT_CRC 0x00
#define WITH_CRC 0x01
#define ADD_CRC 0x02
#define WITHOUT_CRC_AES 0x08
#define WITH_CRC_AES 0x09
#define ADD_CRC_AES 0x0A
#define MODE_RX 0x80
#define MODE_RX_AES 0xC0
#define N_MAX                            48
#define TIMEOUT_ms			 100
#define MESSAGE_LENGTH  40

unsigned char ucSt[N_MAX];

LoraIno sx1276;

AES aes ;
byte *key = (unsigned char*)"0123456789010123";
unsigned long long int my_iv = 36753562;

bool bRxEncrypt,bTxEncrypt;

void vShowDecrypted(byte* cipher,int len){
  int bits = 128,i; // encryption robustness
  byte iv [N_BLOCK],deCipher[N_MAX];

  aes.iv_inc();
  aes.set_IV(my_iv);
  aes.get_IV(iv);
  aes.do_aes_decrypt(cipher,N_MAX,deCipher,key,bits,iv);       
  if(deCipher[1] < N_MAX+2)
    for(i = 0; i < deCipher[1]+2 ; i++) 
      Serial.write(deCipher[i]);
}

void vAnswer(byte* ucSt,int len){
  int bits = 128,i; // encryption robustness
  byte iv [N_BLOCK],cipher[N_MAX];

  if( bTxEncrypt ){
    //for(i = -2 ; i < len ; i++)
    //  Serial.write(ucSt[i]);
    //delay(2);
    aes.iv_inc();
    aes.set_IV(my_iv);
    aes.get_IV(iv);
    aes.do_aes_encrypt(ucSt-2,N_MAX,cipher,key,bits,iv);
    //delay(2000);
    //for(i = 0 ; i < 48 ; i++)
    //  Serial.write(cipher[i]);
    //delay(2000);
    //vShowDecrypted(cipher,48);

    sx1276.mode_tx(cipher,N_MAX);
  }else
    sx1276.mode_tx(ucSt,len);
}

void vAnswerCheckingCRC(byte* ucSt,int len){
  int bits = 128; // encryption robustness
  byte iv [N_BLOCK],cipher[N_MAX];
  unsigned int uiCRC = sx1276.uiModRTU_CRC(ucSt,len-2);

  if((byte)(uiCRC >> 8) == ucSt[len-1] && (byte)(uiCRC & 0xFF) == ucSt[len-2]){ 
    if( bTxEncrypt ){
      aes.iv_inc();
      aes.set_IV(my_iv);
      aes.get_IV(iv);
      aes.do_aes_encrypt(ucSt-2,N_MAX,cipher,key,bits,iv);
      sx1276.mode_tx(cipher,N_MAX);
    }else
      sx1276.mode_tx(ucSt,len);
  }else{
    Serial.println("   CRC doesn't match   ");
  }
}

void vAnswerAddingCRC(byte* ucSt,int len){
  int bits = 128; // encryption robustness
  byte iv [N_BLOCK],cipher[N_MAX];
  unsigned int uiCRC = sx1276.uiModRTU_CRC(ucSt,len);
  
  ucSt[len] = (byte)(uiCRC & 0xFF);
  ucSt[len+1] = (byte)(uiCRC >> 8);
  
  if( bTxEncrypt ){
    aes.iv_inc();
    aes.set_IV(my_iv);
    aes.get_IV(iv);
    aes.do_aes_encrypt(ucSt-2,N_MAX,cipher,key,bits,iv);
    sx1276.mode_tx(cipher,N_MAX);
  }else
    sx1276.mode_tx(ucSt,len+2);
}

void vProcessa(byte* ucSt,int len){
  byte byMSB,byMid,byLSB;
  
  if(ucSt[0] == 0x20 && ucSt[1] == 0x00){
    digitalWrite(TXEN,LOW); // Disabling tx antenna
    digitalWrite(RXEN,LOW); // Disabling rx antenna
    sx1276.bRxMode = false;
    sx1276.bTxMode = false;
    sx1276.readFreq(&byMSB,&byMid,&byLSB);
    Serial.write(0x20);
    Serial.write(0x03);
    Serial.write(byMSB);
    Serial.write(byMid);
    Serial.write(byLSB);
    return;
  }
  if(ucSt[0] == 0x10 && ucSt[1] == 0x03){
    digitalWrite(TXEN,LOW); // Disabling tx antenna
    digitalWrite(RXEN,LOW); // Disabling rx antenna
    sx1276.bRxMode = false;
    sx1276.bTxMode = false;
    sx1276.Config_SX1276(ucSt[2],ucSt[3],ucSt[4]);
    return;
  } 
  if( ucSt[0] & MODE_RX ){            // 1st byte is function mode
    Serial.println("    Rx mode");
    sx1276.bRxMode = true;
    sx1276.bTxMode = false;
    if( (ucSt[0] & MODE_RX_AES) == MODE_RX_AES)
      bRxEncrypt = true;
    else
      bRxEncrypt = false;    
  }else{
    if(ucSt[1] == len-2){           // 2nd byte is payload length
      sx1276.bRxMode = false;
      sx1276.bTxMode = true;

      switch(ucSt[0]){
        case WITHOUT_CRC:
             bTxEncrypt = false;
             vAnswer(ucSt+2,ucSt[1]);
             break;
        case WITHOUT_CRC_AES:
             bTxEncrypt = true;
             vAnswer(ucSt+2,ucSt[1]);
             break;
        case WITH_CRC:
             bTxEncrypt = false;
             vAnswerCheckingCRC(ucSt+2,ucSt[1]);
             break;
        case WITH_CRC_AES:
             bTxEncrypt = true;
             vAnswerCheckingCRC(ucSt+2,ucSt[1]);
             break;
        case ADD_CRC:
             bTxEncrypt = false;
             vAnswerAddingCRC(ucSt+2,ucSt[1]);
             break;
        case ADD_CRC_AES:
             bTxEncrypt = true;
             vAnswerAddingCRC(ucSt+2,ucSt[1]);
             break;
        default:
             vAnswer(ucSt,len);
     }
    }else
      Serial.println("Length doesn't match");
  }
}

void vReadLoraPacket(){
  int i,bits = 128; // encryption robustness
  byte iv [N_BLOCK] ;
  byte check[N_MAX] ;

  if(sx1276.bReadLora(ucSt,MESSAGE_LENGTH)){
	if (bRxEncrypt){
	  Serial.write(ucSt[0]);
	  Serial.write(ucSt[1]);
	  vShowDecrypted(ucSt+2,48);
	}else{
	  for(i = 0; i < ucSt[1] ; i++) 
	    Serial.write(ucSt[i]);
	}
  }
}

void setup() {
  Serial.begin(9600);
  sx1276.init_SX1276();
  bRxEncrypt = bTxEncrypt = false;
}

void loop() {
  int i,nCmpt = 0;

  digitalWrite(LED1,LOW); 
  digitalWrite(LED2,LOW);  
  sx1276.reset_sx1276(); 
  sx1276.Config_SX1276();
  
  while(1){
    while(Serial.available() > 0) {
      ucSt[nCmpt] = (unsigned char)Serial.read();
      nCmpt++;
      delay(2);
    }
    if(nCmpt){
      vProcessa(ucSt,nCmpt);
    }else{
      if(sx1276.bRxMode){
          vReadLoraPacket();
      }
    }
    nCmpt = 0;
  }     
}
