#include "loraIno.h"

bool LoraIno::bReadLora(byte* byReceivedPacket,int nQuants){
  unsigned char temp,payload_size,i;
  unsigned long ulPreviousMillis = millis();
  unsigned int uiLenModbusWithoutCRC,uiCRC;

  init_rx();                                       // rx mode
  //Serial.println("\n\rStarting to read LoRa");
  while(1){
    temp=SPIreadRegister(LR_RegIrqFlags);                 // read interupt
    if(temp & 0x40){                                        // wait for rxdone flag
      SPIwriteRegister(LR_RegIrqFlags,0xff);            // clear interupt
      temp = SPIreadRegister(LR_RegFifoRxCurrentaddr);  // read RxCurrentaddr
      SPIwriteRegister(LR_RegFifoAddrPtr,temp);   // RxCurrentaddr -> FiFoAddrPtr
      
      payload_size = SPIreadRegister(LR_RegRxNbBytes);  // read packet size
      //SPIreadBurst(0x00, rxbuf, MESSAGE_LENGTH);          // read from fifo
      SPIreadBurst(0x00, byReceivedPacket, nQuants);          // read from fifo

	  /*	
      Serial.println("--------- { RX ------------");     
      Serial.print("cmpt: ");Serial.print((char)check[0]);
      Serial.print(" ; random RX: ");Serial.print((char)check[1]);
      Serial.print(" ; modbus length: ");Serial.println((int)check[2]);
      uiLenModbusWithoutCRC = check[2];
      uiCRC = uiModRTU_CRC(&check[3],uiLenModbusWithoutCRC);
      if((check[uiLenModbusWithoutCRC+3] == (byte)(uiCRC & 0xFF))
          &&     (check[uiLenModbusWithoutCRC+4] == (uiCRC >> 8))){      
        //for(i = 3 ; i < 3+check[2]+2 ; i++){
        //  Serial.print(check[i],HEX);
        //  Serial.print(" ");
        //}
      }else{
        //Serial.print("BAD CRC");
        return false;
        //for(i = 3 ; i < 14 ; i++){
        //  Serial.print(check[i],HEX);
        //  Serial.print(" ");
        //}
      }
      //Serial.println("\n\r=========   RX } ==========");
      //Serial.println("\n\rLoRa has been read");
      */
      return true;
    }
    if(millis() - ulPreviousMillis > TIMEOUT_ms){
      //Serial.println("LoRa Time out");
      return false;
    }
  }
}

void LoraIno::mode_tx(byte* byTextToSend,int nQuants) 
{
    unsigned char addr,temp;
    int i;
    static char cTrama = '0';
   
    cTrama++;
    if( cTrama > '9' ) cTrama = '0';
    
    for(i = nQuants-1 ; i >= 0 ; i--)
      byTextToSend[i+2] = byTextToSend[i];
    
    byTextToSend[0] = cTrama; // Canvia de '0' a '9'
    nQuants += 2;
    byTextToSend[1] = nQuants;    

    digitalWrite(TXEN,HIGH);                                // open tx antenna switch
    digitalWrite(RXEN,LOW); 
  
	SPIwriteRegister(REG_LR_DIOMAPPING1,0x41); 		// DIO0=TxDone,DIO1=RxTimeout,DIO3=ValidHeader
		
	SPIwriteRegister(LR_RegIrqFlags,0xff);		// clearing interupt
	SPIwriteRegister(LR_RegIrqFlagsMask,0xf7);		// enabling txdone
	SPIwriteRegister(LR_RegPayloadLength,nQuants);	// payload length

	addr = SPIreadRegister(LR_RegFifoTxBaseAddr);	// read TxBaseAddr        
	SPIwriteRegister(LR_RegFifoAddrPtr,addr);	// TxBaseAddr->FifoAddrPtr          
	
	SPIwriteBurst(0x00,byTextToSend,nQuants);	// write data in fifo
	SPIwriteRegister(LR_RegOpMode,0x03);			// mode tx, high frequency
	     
	digitalWrite(LED1, !digitalRead(LED1));
        
	temp=SPIreadRegister(LR_RegIrqFlags);		// read interput flag
	while(!(temp&0x08))					// wait for txdone flag
	{
		temp=SPIreadRegister(LR_RegIrqFlags);
		               
	}
	digitalWrite(TXEN,LOW);                                 // close tx antenna switch
	digitalWrite(RXEN,LOW); 
  
	SPIwriteRegister(LR_RegIrqFlags,0xff);		// clearing interupt
	SPIwriteRegister(LR_RegOpMode,0x01);  		// standby mode, high frequency
}

void LoraIno::init_rx(void)
{
	unsigned char addr; 
	
        digitalWrite(TXEN,LOW);                                 // open rx antenna switch
        digitalWrite(RXEN,HIGH);
			    
	SPIwriteRegister(REG_LR_DIOMAPPING1,0x01);		//DIO0=00, DIO1=00, DIO2=00, DIO3=01  DIO0=00--RXDONE
	  
	SPIwriteRegister(LR_RegIrqFlagsMask,0x3f);		// enable rxdone and rxtimeout
	SPIwriteRegister(LR_RegIrqFlags,0xff);		// clearing interupt

	addr = SPIreadRegister(LR_RegFifoRxBaseAddr);	// read RxBaseAddr
	SPIwriteRegister(LR_RegFifoAddrPtr,addr);	// RxBaseAddr->FifoAddrPtr
	SPIwriteRegister(LR_RegOpMode,0x05);			// rx mode continuous high frequency}
}

// Compute the MODBUS RTU CRC
// Adapted to Arduino from http://www.ccontrolsys.com/w/How_to_Compute_the_Modbus_RTU_Message_CRC
unsigned int LoraIno::uiModRTU_CRC(byte* buf, int len){
  unsigned int crc = 0xFFFF;
 
  for (int pos = 0; pos < len; pos++) {
    crc ^= (unsigned int)buf[pos];          // XOR byte into least sig. byte of crc
 
    for (int i = 8; i != 0; i--) {    // Loop over each bit
      if ((crc & 0x0001) != 0) {      // If the LSB is set
        crc >>= 1;                    // Shift right and XOR 0xA001
        crc ^= 0xA001;
      }
      else                            // Else LSB is not set
        crc >>= 1;                    // Just shift right
    }
  }
  // Note, this number has low and high bytes swapped, so use it accordingly (or swap bytes)
  return crc;  
}

void LoraIno::readFreq(byte* byMsb,byte* byMid,byte* byLsb){
  *byMsb = SPIreadRegister(LR_RegFrMsb);
  *byMid = SPIreadRegister(LR_RegFrMid);
  *byLsb = SPIreadRegister(LR_RegFrLsb);
}

void LoraIno::Config_SX1276(byte byMsb,byte byMid,byte byLsb)
{
	init_SX1276();
	reset_sx1276();
	// put in sleep mode to configure

	SPIwriteRegister(LR_RegOpMode,0x00);		// sleep mode, high frequency
	delay(10);

	SPIwriteRegister(REG_LR_TCXO,0x09);		// external crystal
        SPIwriteRegister(LR_RegOpMode,0x80);		// LoRa mode, high frequency
        
        SPIwriteRegister(LR_RegFrMsb,byMsb);
        SPIwriteRegister(LR_RegFrMid,byMid);
        SPIwriteRegister(LR_RegFrLsb,byLsb);         
        //SPIwriteRegister(LR_RegFrMsb,0xD8);
        //SPIwriteRegister(LR_RegFrMid,0x59);
        //SPIwriteRegister(LR_RegFrLsb,0xBE);   // frequency：868.000 MHz  (doble de 434, si MSB de 434 és 6c per a 868 suposo EC)      
        // 0xD859BE = 14178950 --> x 61,035 dona 865400000   


	SPIwriteRegister(LR_RegPaConfig,0xFF);	// max output power PA_BOOST enabled
	//SPIwriteRegister(LR_RegPaConfig,0xCE);	// max output power PA_BOOST enabled


	SPIwriteRegister(LR_RegOcp,0x0B);		// close over current protection  (ocp)
	//SPIwriteRegister(LR_RegOcp,0x2B);		// close over current protection  (ocp)
	SPIwriteRegister(LR_RegLna,0x23);		// Enable LNA
	//SPIwriteRegister(LR_RegLna,0x00);		// Enable LNA

        SPIwriteRegister(LR_RegModemConfig1,0x72);	// signal bandwidth：125kHz,error coding= 4/5, explicit header mode
        //SPIwriteRegister(LR_RegModemConfig1,0x73);	// signal bandwidth：125kHz,error coding= 4/5, explicit header mode
        
	SPIwriteRegister(LR_RegModemConfig2,0xC7);	// spreading factor：12
	//SPIwriteRegister(LR_RegModemConfig2,0xC0);	// spreading factor：12

	SPIwriteRegister(LR_RegModemConfig3,0x08);	// LNA? optimized for low data rate
	//SPIwriteRegister(LR_RegModemConfig3,0x03);	// LNA? optimized for low data rate
	  
	SPIwriteRegister(LR_RegSymbTimeoutLsb,0xFF);     // max receiving timeout

	SPIwriteRegister(LR_RegPreambleMsb,0x00);
	SPIwriteRegister(LR_RegPreambleLsb,16);          // preamble 16 bytes  	
	//SPIwriteRegister(LR_RegPreambleLsb,0x10);          // preamble 16 bytes  	

	SPIwriteRegister(REG_LR_PADAC,0x87);             // transmission power 20dBm
	SPIwriteRegister(LR_RegHopPeriod,0x00);          // no frequency hoping

	SPIwriteRegister(REG_LR_DIOMAPPING2,0x01);       // DIO5=ModeReady,DIO4=CadDetected
	SPIwriteRegister(LR_RegOpMode,0x01);             // standby mode, high frequency
}

void LoraIno::Config_SX1276(void){
	Config_SX1276(0xD8,0x59,0xBE);
    // 0xD859BE = 14178950 --> x 61,035 dona 865400000   
}


void LoraIno::reset_sx1276(void)
{
    digitalWrite(TXEN, LOW);
    digitalWrite(RXEN, LOW);	

    digitalWrite(NRESET, LOW);	
    delay(10);
    digitalWrite(NRESET, HIGH);	
    delay(20);    
} 

void LoraIno::SPIreadBurst(unsigned char addr, unsigned char *ptr, unsigned char len)
{
	unsigned char i;
	unsigned char result;
        
        digitalWrite(SS, LOW);          // Select LoRa module
        
        SPDR = addr;                // Send address & Start transmission. In READ mode bit 7 of address is always 0! for sx1276
        while (!(SPSR & (1<<SPIF)))     // Wait for transmission to finish
        {
        };
        result = SPDR;               // Discard first reading
        
        for (i=0; i <= len; i++){
           
           SPDR = 0;                     // Sending dummy byte to get the result
           while (!(SPSR & (1<<SPIF)))   // Wait for transmission to finish
           {
           };
           *ptr = SPDR;               // move pointer
           
           
           ptr++;
        } 
        digitalWrite(SS, HIGH);         // Deselect LoRa module	
}

void LoraIno::SPIwriteBurst(unsigned char addr, unsigned char *ptr, unsigned char len)
{ 
	unsigned char i;
	unsigned char result;
        
        digitalWrite(SS, LOW);          // Select LoRa module
        
        SPDR = addr | 0x80;              // Send address & Start transmission. In WRITE mode bit 7 of address is always 1! for sx1276
        while (!(SPSR & (1<<SPIF)))     // Wait for transmission to finish
        {
        };
        result = SPDR;               // Discard first reading
        
        for (i=0; i <= len; i++){
           
           SPDR = *ptr;                     // Sending bytes 
           while (!(SPSR & (1<<SPIF)))     // Wait for transmission to finish
           {
           };
           result = SPDR;               // Discard second reading  
           ptr++;
        } 
        digitalWrite(SS, HIGH);         // Deselect LoRa module
}


byte LoraIno::SPIwriteRegister(byte addr,byte value) {
  byte result;
  
  digitalWrite(SS, LOW);          // Select LoRa module
  
  SPDR = addr | 0x80;              // Send address & Start transmission. In WRITE mode bit 7 of address is always 1! for sx1276
  while (!(SPSR & (1<<SPIF)))     // Wait for transmission to finish
  {
  };
  result = SPDR;                   // Discard first reading
  
  
  SPDR = value;                     // Sending byte 
  while (!(SPSR & (1<<SPIF)))       // Wait for transmission to finish
  {
  };
  result = SPDR;                   // Discard second reading
  
  digitalWrite(SS, HIGH);         // Deselect LoRa module
    
}

byte LoraIno::SPIreadRegister(byte addr) { 
  byte result;
  
  digitalWrite(SS, LOW);          // Select LoRa module
  
  SPDR = addr;                    // Send address & Start transmission. In READ mode bit 7 of address is always 0! for sx1276
  while (!(SPSR & (1<<SPIF)))     // Wait for transmission to finish
  {
  };
  result = SPDR;               // Discard first reading
  
  SPDR = 0x0;                     // Sending dummy byte to get the result
  while (!(SPSR & (1<<SPIF)))     // Wait for transmission to finish
  {
  };
  result = SPDR;               // Reading register value
  
  digitalWrite(SS, HIGH);         // Deselect LoRa module
  
  return (result);
  
}

int LoraIno::nCmpt(byte *sz){
  int n=0;
  
  while(*sz){
    sz++; n++;
  }
  
  return n;
}

void LoraIno::init_SX1276(){
  byte temp = 0;  

  // Initializing SPI pins
  pinMode(MOSI, OUTPUT);
  pinMode(MISO, INPUT);
  pinMode(SCK,OUTPUT);
  pinMode(SS,OUTPUT);
  digitalWrite(SS,HIGH); //disabling LoRa module
  
  // Initializing other I/O pins
  pinMode(NRESET, OUTPUT);
  pinMode(TXEN, OUTPUT);
  pinMode(RXEN, OUTPUT);
  pinMode(LED1, OUTPUT);
  pinMode(LED2, OUTPUT);
  
  digitalWrite(NRESET,HIGH); // Deassert reset
  digitalWrite(TXEN,LOW); // Disabling tx antenna
  digitalWrite(RXEN,LOW); // Disabling rx antenna
  digitalWrite(LED1,LOW); 
  digitalWrite(LED2,LOW); 
  
  /* Initializing SPI registers
  description of every SPCR register bits
  | 7    | 6    | 5    | 4    | 3    | 2    | 1    | 0    |
  | SPIE | SPE  | DORD | MSTR | CPOL | CPHA | SPR1 | SPR0 |

  SPIE - Enable SPI interupt (logic 1)
  SPE  - Enable SPI (logic 1)
  DORD - Send Least Significant Bit (LSB) first (logic 1) , Send Most Significant Bit (MSB) first (logic 0) 
  MSTR - Enable SPI master mode (logic 1), slave mode (logic 0)
  CPOL - Setup clock signal inactive in high (logic 1), inactive in low (logic 0)
  CPHA - Read data on Falling Clock Edge (logic 1), Rising edge (logic 0)
  SPR1 y SPR0 - Setup SPI data rate: 00 Fastest (4MHz), 11 Slowest (250KHz)
  
  // SPCR = 01010011
  //interupt disabled,spi enabled,most significant bit (msb) first,SPI master,clock inactive low,
  data fech rising clock edge, slowest data rate*/

  SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR1)|(1<<SPR0);
  temp=SPSR;  //Reading and discarding previous data
  temp=SPDR;  //Reading and discarding previous data
  delay(10);
  
  bTxMode = bRxMode = false; 
}

LoraIno::LoraIno(){
  bTxMode = bRxMode = false; 
}
