#include <ESP8266WiFi.h>

#ifdef ESP8266
  extern "C" {
    #include "user_interface.h"
  }
  #include "spiESP.h"
  #include "spiEspDirect.h"
  #include "loraESP.h"
#endif

#include <AES.h>

#define WITHOUT_CRC 0x00
#define WITH_CRC 0x01
#define ADD_CRC 0x02
#define WITHOUT_CRC_AES 0x08
#define WITH_CRC_AES 0x09
#define ADD_CRC_AES 0x0A
#define MODE_RX 0x80
#define MODE_RX_AES 0xC0
#define N_MAX 64
#define TIMEOUT_ms       100
#define MESSAGE_LENGTH  40

unsigned char ucSt[N_MAX];

SpiEsp spiESP;
SpiEspDirect spiEspDirect(&spiESP);
LoRaESP sx1276(&spiEspDirect);

AES aes ;
byte *key = (unsigned char*)"0123456789010123";
unsigned long long int my_iv = 36753562;

bool bRxEncrypt,bTxEncrypt;

void vShowDecrypted(byte* cipher,int len){
  int bits = 128,i; // encryption robustness
  byte iv [N_BLOCK],deCipher[N_MAX];

  aes.iv_inc();
  aes.set_IV(my_iv);
  aes.get_IV(iv);
  aes.do_aes_decrypt(cipher,N_MAX,deCipher,key,bits,iv);       
  if(deCipher[1] < N_MAX+2)
    for(i = 0; i < deCipher[1]+2 ; i++) 
      Serial.write(deCipher[i]);
}

void vAnswer(byte* ucSt,int len){
  int bits = 128,i; // encryption robustness
  byte iv [N_BLOCK],cipher[N_MAX];

  if( bTxEncrypt ){
    aes.iv_inc();
    aes.set_IV(my_iv);
    aes.get_IV(iv);
    aes.do_aes_encrypt(ucSt-2,N_MAX,cipher,key,bits,iv);
    sx1276.mode_tx(cipher,N_MAX);
  }else
    sx1276.mode_tx(ucSt,len);
}

void vAnswerCheckingCRC(byte* ucSt,int len){
  int bits = 128; // encryption robustness
  byte iv [N_BLOCK],cipher[N_MAX];
  unsigned int uiCRC = sx1276.uiModRTU_CRC(ucSt,len-2);

  if((byte)(uiCRC >> 8) == ucSt[len-1] && (byte)(uiCRC & 0xFF) == ucSt[len-2]){ 
    if( bTxEncrypt ){
      aes.iv_inc();
      aes.set_IV(my_iv);
      aes.get_IV(iv);
      aes.do_aes_encrypt(ucSt-2,N_MAX,cipher,key,bits,iv);
      sx1276.mode_tx(cipher,N_MAX);
    }else
      sx1276.mode_tx(ucSt,len);
  }else{
    Serial.println("   CRC doesn't match   ");
  }
}

void vAnswerAddingCRC(byte* ucSt,int len){
  int bits = 128; // encryption robustness
  byte iv [N_BLOCK],cipher[N_MAX];
  unsigned int uiCRC = sx1276.uiModRTU_CRC(ucSt,len);
  
  ucSt[len] = (byte)(uiCRC & 0xFF);
  ucSt[len+1] = (byte)(uiCRC >> 8);
  
  if( bTxEncrypt ){
    aes.iv_inc();
    aes.set_IV(my_iv);
    aes.get_IV(iv);
    aes.do_aes_encrypt(ucSt-2,N_MAX,cipher,key,bits,iv);
    sx1276.mode_tx(cipher,N_MAX);
  }else
    sx1276.mode_tx(ucSt,len+2);
}

void vProcessa(byte* ucSt,int len){  
  byte byMSB,byMid,byLSB;
  /*////////////*
  int i = 0;
  while(i < len){
     Serial.write(ucSt[i]);
     i++;
  }
  *///////////////
  if(ucSt[0] == 0x20 && ucSt[1] == 0x00){
    sx1276.vDisablingAntennaTxRx();
    sx1276.bRxMode = false;
    sx1276.bTxMode = false;
    sx1276.readFreq(&byMSB,&byMid,&byLSB);
    Serial.write(0x20);
    Serial.write(0x03);
    Serial.write(byMSB);
    Serial.write(byMid);
    Serial.write(byLSB);
    return;
  }
  if(ucSt[0] == 0x10 && ucSt[1] == 0x03){
    sx1276.vDisablingAntennaTxRx();
    sx1276.bRxMode = false;
    sx1276.bTxMode = false;
    sx1276.Config_SX1276(ucSt[2],ucSt[3],ucSt[4]);
    return;
  } 
  if( ucSt[0] & MODE_RX ){            // 1st byte is function mode
    Serial.println("    Rx mode");
    sx1276.bRxMode = true;
    sx1276.bTxMode = false;
    if( (ucSt[0] & MODE_RX_AES) == MODE_RX_AES)
      bRxEncrypt = true;
    else
      bRxEncrypt = false;    
  }else{
    if(ucSt[1] == len-2){           // 2nd byte is payload length
      sx1276.bRxMode = false;
      sx1276.bTxMode = true;

      switch(ucSt[0]){
        case WITHOUT_CRC:
             bTxEncrypt = false;
             vAnswer(ucSt+2,ucSt[1]);
             break;
        case WITHOUT_CRC_AES:
             bTxEncrypt = true;
             vAnswer(ucSt+2,ucSt[1]);
             break;
        case WITH_CRC:
             bTxEncrypt = false;
             vAnswerCheckingCRC(ucSt+2,ucSt[1]);
             break;
        case WITH_CRC_AES:
             bTxEncrypt = true;
             vAnswerCheckingCRC(ucSt+2,ucSt[1]);
             break;
        case ADD_CRC:
             bTxEncrypt = false;
             vAnswerAddingCRC(ucSt+2,ucSt[1]);
             break;
        case ADD_CRC_AES:
             bTxEncrypt = true;
             vAnswerAddingCRC(ucSt+2,ucSt[1]);
             break;
        default:
             vAnswer(ucSt,len);
     }
    }else
      Serial.println("Length doesn't match");
  }
}

void vReadLoraPacket(){
  int i,bits = 128; // encryption robustness
  byte iv [N_BLOCK] ;
  byte check[N_MAX] ;

  if(sx1276.bReadLora(ucSt,MESSAGE_LENGTH)){
    if (bRxEncrypt){
      Serial.write(ucSt[0]);
      Serial.write(ucSt[1]);
      vShowDecrypted(ucSt+2,48);
    }else{
      for(i = 0; i < ucSt[1] ; i++) 
        Serial.write(ucSt[i]);
    }
  }
}

void setup() {
  Serial.begin(9600);
  sx1276.init_SX1276();
  bRxEncrypt = bTxEncrypt = false;
}

void loop() {
  int i,nCmpt = 0;

  //digitalWrite(LED1,LOW); 
  //digitalWrite(LED2,LOW);  
  sx1276.reset_sx1276(); 
  sx1276.Config_SX1276();
  
  while(1){
    yield();
    while(Serial.available() > 0) {
      ucSt[nCmpt] = (unsigned char)Serial.read();
      nCmpt++;
      delay(2);
    }
    if(nCmpt){
      vProcessa(ucSt,nCmpt);
    }else{
      if(sx1276.bRxMode){
          vReadLoraPacket();
      }
    }
    nCmpt = 0;
  }      
}

